#include <SoftwareSerial.h>

//#include <Sonar_srf08.h>

#include <Wire.h>

#include <PS2X_lib.h>

#include <pins_arduino.h>

#if defined(UBRR1H)
#define SSCSerial         Serial1
#else
#define cSSC_IN     11
#define cSSC_OUT     10

 SoftwareSerial SSCSerial(cSSC_IN, cSSC_OUT);
#endif


//===========================================================================================
// Botboarduino JR Test program
//===========================================================================================
// First lets try to blink our LEDS.
int    iPin;
int    iVL1;
int    iVL2;
byte    bTestMode;                // which things are we testing?
int    aiVals[3];                // for 3 buttons

#define SOUND_PIN    5
extern void MSound(uint8_t _pin, byte cNotes, ...);


void setup() {
    int i;
    int j;
    delay(250);
    Serial.begin(38400);    
    Serial.println("Botboarduino Test program");
    
    iPin = 2;     // first pin we will update in loop;

    for (i = 7; i <= 9; i++) {
        pinMode(i, OUTPUT);
        for (j=0; j< 5; j++) {
            digitalWrite(i, LOW);
            delay(100);
            digitalWrite(i, HIGH);
            delay(100);
        }
    }
    
    pinMode(13, OUTPUT);
    for (j=0; j< 5; j++) {
        digitalWrite(13, HIGH);
        delay(100);
        digitalWrite(13, LOW);
        delay(100);
    }

    bTestMode = 0;    
    aiVals[0] = HIGH;                // for 3 buttons
    aiVals[1] = HIGH;                // for 3 buttons
    aiVals[2] = HIGH;                // for 3 buttons
}

void loop() {
    int ich;
    
    bTestMode = 0;
    Serial.println("Current Tests:");
    Serial.println("1 - Digital Pin Test");
    Serial.println("2 - Analog Pin Test");
    Serial.println("3 - Serial SSC32 Test");
    Serial.println("4 - Sound Test");
    Serial.println("5 - I2c/SRF08 Test");
    Serial.println("6 - PS2 Test");
    Serial.print("Choose: ");
    while (bTestMode == 0) {
        // Check our three buttons. 
        CheckButtons();
        ich = Serial.read();
        if ((ich >= '1') && (ich <= '9')) {
            bTestMode = ich - '0';   
        }
    }
    switch (bTestMode) {
            break;
        case 1:
            DigitalPinTest();
            break;
        case 2:
            AnalogPinTest();
            break;
        case 3:
            SSCTest();
            break;
        case 4:
            SoundTest();
            break;
        case 5:
            I2CTest();
            break;
        case 6:
            PS2Test();
        default:
            break;            
    }
            
}
//==============================================================================
// CheckButtons
//==============================================================================
void CheckButtons(void) {
    int i;
    int iVal;
    for (i=7; i<=9; i++ ) {
        pinMode(i, INPUT);
        iVal = digitalRead(i);
        if (aiVals[i-7] != iVal) {
            aiVals[i-7] = iVal;
            Serial.print("Button ");
            Serial.print (i, DEC);
            if (iVal) 
                Serial.println("Released");
            else
                Serial.println("Pressed");
        }
    }
            
}

//==============================================================================
// Digital pin test
//==============================================================================
void DigitalPinTest() {
   int i;


    for (iPin=2; iPin <= 13; iPin++) {        
        pinMode(iPin, OUTPUT);
        digitalWrite(iPin, LOW);
    }     
    delay(10);
    
    for (iPin=2; iPin <= 13; iPin++) {        
        if ((iPin % 5) == 0) {
            i = iPin / 5;
            while (i--) {
                digitalWrite(iPin, HIGH);
                delay(10);
                digitalWrite(iPin, LOW);
                delay(10);
            }
        }
        digitalWrite(iPin, HIGH);
        delay(25);
        digitalWrite(iPin, LOW);
    }     

    for (iPin = A0; iPin <= A7; iPin++) {
        pinMode(iPin, OUTPUT);
        if ((iPin % 5) == 0) {
            i = iPin / 5;
            while (i--) {
                digitalWrite(iPin, HIGH);
                delay(10);
                digitalWrite(iPin, LOW);
                delay(10);
            }
        }
        digitalWrite(iPin, HIGH);
        delay(25);
        digitalWrite(iPin, LOW);
    }
}
//==============================================================================
// Analog pin test
//==============================================================================
void AnalogPinTest() {
   int i;
   int iVal;
   int iValPrev;

   Serial.println("Analog test - press button to exit");

    for (;;) {
        for (i=7; i<=9; i++ ) {
            pinMode(i, INPUT);
            iVal = digitalRead(i);
            if (!iVal)
                return;
        }
    
        iVal = analogRead(0);
        if (iVal != iValPrev) {
            iValPrev = iVal;
            Serial.println(iVal);
        }
    }
            
}
        
//==============================================================================
// SSCTest - Quick and dirty test to see if We have bidirectional communications
// setup...
//==============================================================================
void  SSCTest() {
    // Try output of Ver command to SSCSerialand see if we get a response. 
    byte abT[80];
    byte cbRead;
    Serial.println("SSC-32 Serial Ver test at 38400");
    SSCSerial.begin(38400);
    delay (10);
    SSCSerial.println("ver");
    cbRead = CommReadLn(&SSCSerial, abT, sizeof(abT), 1000, 13);
    if (cbRead) {
        Serial.print("SSCSerial : ");
        Serial.write(abT, cbRead);
        Serial.println("");
    } else {
        Serial.print("SSCSerial : *** No Response ***");
    }    

}

//==============================================================================
// SoundTest - Simple try to play some sounds
//==============================================================================
void SoundTest(void) {
    MSound(SOUND_PIN, 3, 60, 2000, 80, 2250, 100, 2500);
}


//==============================================================================
// I2CTest - Quick and dirty test to see if I2c works.  Trying with an SRF08 or
//            SRF10.
//==============================================================================
void  I2CTest() {
#ifdef LATER  
    Sonar_srf08 MySonar;

#define CommandRegister 0x00
    int New_Address = 248; //0xF8
#define ResultRegister  0x02

    int DEBUG = 1;
    char unit = 'i'; // 'i' for inches , 'c' for centimeters
    int sensorReading;
    int sensorReadingPrev = 0;
    
    // setup code
     MySonar.connect();  
     MySonar.changeAddress(CommandRegister, New_Address);
     New_Address += 4; 

     while (digitalRead(9)) {        // Will loop until button is pressed
         // set units for reading out distance
         MySonar.setUnit(CommandRegister, New_Address, unit);
         delay(70);
        
         // set register for reading
         MySonar.setRegister(New_Address, ResultRegister);
         // read data from result register
         sensorReading = MySonar.readData(New_Address, 2);
         //print out distance
         if (sensorReading != sensorReadingPrev) {
              Serial.print("Distance: ");
              Serial.print(sensorReading);
              Serial.print(" inches");
              Serial.println();
              sensorReadingPrev = sensorReading;
         }
     }   
#endif   
}

//==============================================================================
// PS2Test - Quick and dirty test to see if I2c works.  Trying with an SRF08 or
//            SRF10.
//==============================================================================
void  PS2Test() {

    PS2X ps2x;
    byte bRet;
    
    bRet = ps2x.config_gamepad(9, 7, 8, 6);  // configure no pressure or motors...
    if(bRet == 0)
       Serial.println("Found Controller, configured successful");
    else if(bRet == 1)
       Serial.println("No controller found, check wiring, see readme.txt to enable debug. visit www.billporter.info for troubleshooting tips");
   else if(bRet == 2)
       Serial.println("Controller found but not accepting commands. see readme.txt to enable debug. Visit www.billporter.info for troubleshooting tips");
   else if(bRet == 3)
       Serial.println("Controller refusing to enter Pressures mode, may not support it. ");
    bRet = ps2x.readType(); 
    switch(bRet) {
        case 0:
            Serial.println("Unknown Controller type");
            break;
        case 1:
            Serial.println("DualShock Controller Found");
            break;
       case 2:
            Serial.println("GuitarHero Controller Found");
           break;
     }

    
    
     while (digitalRead(9)) {        // Will loop until button is pressed
         // set units for reading out distance
         ps2x.read_gamepad();          //read controller 
         Serial.print(ps2x.ButtonDataByte(), HEX);
         Serial.print(":");
         Serial.print(ps2x.Analog(PSS_LY), DEC); //Left stick, Y axis. Other options: LX, RY, RX  
         Serial.print(",");
         Serial.print(ps2x.Analog(PSS_LX), DEC); 
         Serial.print(",");
         Serial.print(ps2x.Analog(PSS_RY), DEC); 
         Serial.print(",");
         Serial.println(ps2x.Analog(PSS_RX), DEC); 
        delay(200);
     }   
   
}

    
//==============================================================================
//    SoundNoTimer - Quick and dirty tone function to try to output a frequency
//            to a speaker for some simple sounds.
//==============================================================================
void SoundNoTimer(uint8_t _pin, unsigned long duration,  unsigned int frequency)
{
    volatile uint8_t *pin_port;
    volatile uint8_t pin_mask;
    long toggle_count = 0;
    long lusDelayPerHalfCycle;
    
    // Set the pinMode as OUTPUT
    pinMode(_pin, OUTPUT);

    pin_port = portOutputRegister(digitalPinToPort(_pin));
    pin_mask = digitalPinToBitMask(_pin);
    
    toggle_count = 2 * frequency * duration / 1000;
    lusDelayPerHalfCycle = 1000000L/(frequency * 2);
    
    // if we are using an 8 bit timer, scan through prescalars to find the best fit
    while (toggle_count--) {
        // toggle the pin
        *pin_port ^= pin_mask;
        
        // delay a half cycle
        delayMicroseconds(lusDelayPerHalfCycle);
    }    
    *pin_port &= ~(pin_mask);  // keep pin low after stop

}

void MSound(uint8_t _pin, byte cNotes, ...)
{
    va_list ap;
    unsigned int uDur;
    unsigned int uFreq;
    va_start(ap, cNotes);

    while (cNotes > 0) {
        uDur = va_arg(ap, unsigned int);
        uFreq = va_arg(ap, unsigned int);
        SoundNoTimer(_pin, uDur, uFreq);
        cNotes--;
    }
    va_end(ap);
}
    
//=============================================================================
// byte CommReadLn - Read in a buffer of bytes.  We will pass in a timeout
//            value that if we dont receive a character in that amount of time 
//            something is wrong.  
//=============================================================================
// Quick and dirty helper function to read so many bytes in from theXBeeSSC with a timeout and an end of character marker...
#if defined(UBRR1H)
byte CommReadLn(HardwareSerial *pserial, byte *pb, byte cb, word wTimeout, word wEOL)
#else
byte CommReadLn(SoftwareSerial *pserial, byte *pb, byte cb, word wTimeout, word wEOL)
#endif
{
    int ich;
    byte* pbIn = pb;
    unsigned long ulTimeLastChar = millis();

    while (cb) {
        while ((ich = pserial->read()) == -1) {
            // check for timeout
            if ((word)(millis()-ulTimeLastChar) > wTimeout)
                return (byte)(pb-pbIn);
        }
        digitalWrite(9,!digitalRead(9));

        *pb++ = (byte)ich;
        cb--;

        if ((word)ich == wEOL)
            break;    // we matched so get out of here.
        ulTimeLastChar = millis();    // update to say we received something
    }
    return (byte)(pb-pbIn);
}

    
    
